#include <conf.h>

#include <sys/param.h>
#include <sys/malloc.h>

#include <kern/amiga_includes.h>

#if __SASC
/*
 * Change the minimum chuck size of the SASC memory allocator.
 * The default value is 16.384 bytes (SASC6.2), but smaller value
 * is more efficient, since it takes less time to search the free
 * memory pool when it is smaller.
 */
long _MSTEP = 4096;

/*
 * Set the memory type allocated by malloc() to public, since the memory
 * is used by many tasks (including all library users)
 */
unsigned long _MemType = MEMF_PUBLIC;

#endif /* __SASC */

struct SignalSemaphore malloc_semaphore =
{0};
static BOOL initialized = FALSE;

BOOL
malloc_init (void)
{
  if (!initialized)
    {
      /*
     * Initialize malloc_semaphore for use.
     * Do not call bsd_malloc() or bsd_free() before this!
     */
      InitSemaphore (&malloc_semaphore);
      initialized = TRUE;
    }
  return TRUE;
}

void *
bsd_malloc (unsigned long size, int type, int flags)
{
  register void *mem;

  extern void *malloc (size_t);
  extern struct SignalSemaphore malloc_semaphore;

  ObtainSemaphore (&malloc_semaphore);
  mem = malloc ((size_t) size);
  ReleaseSemaphore (&malloc_semaphore);

  return mem;
}

void
bsd_free (void *addr, int type)
{
  extern void free (void *);
  extern struct SignalSemaphore malloc_semaphore;

  ObtainSemaphore (&malloc_semaphore);
  free (addr);
  ReleaseSemaphore (&malloc_semaphore);
}

/*
 * bsd_realloc() cannot be used for reallocating
 * last freed block for obvious reasons
 */

void *
bsd_realloc (void *mem, unsigned long size, int type, int flags)
{
  extern void *realloc (void *, size_t);
  extern struct SignalSemaphore malloc_semaphore;

  ObtainSemaphore (&malloc_semaphore);
  mem = realloc (mem, (size_t) size);
  ReleaseSemaphore (&malloc_semaphore);

  return mem;
}
